/* eslint-disable no-magic-numbers */
import { FIRST_ARRAY_ELEMENT } from '@/constants/common';
import {
  bioEntitiesContentFixture,
  bioEntityContentFixture,
} from '@/models/fixtures/bioEntityContentsFixture';
import { MODELS_MOCK_SHORT } from '@/models/mocks/modelsMock';
import {
  BIOENTITY_INITIAL_STATE_MOCK,
  BIO_ENTITY_LINKING_TO_SUBMAP_DATA_MOCK,
} from '@/redux/bioEntity/bioEntity.mock';
import { DRAWER_INITIAL_STATE } from '@/redux/drawer/drawer.constants';
import { MODELS_INITIAL_STATE_MOCK } from '@/redux/models/models.mock';
import { INITIAL_STORE_STATE_MOCK } from '@/redux/root/root.fixtures';
import { AppDispatch, RootState } from '@/redux/store';
import { getReduxStoreWithActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import { InitialStoreState } from '@/utils/testing/getReduxWrapperWithStore';
import { act, render, screen } from '@testing-library/react';
import { MockStoreEnhanced } from 'redux-mock-store';
import { getTypeBySBOTerm } from '@/utils/bioEntity/getTypeBySBOTerm';
import { BioEntityDrawer } from './BioEntityDrawer.component';

const renderComponent = (
  initialStoreState: InitialStoreState = {},
): { store: MockStoreEnhanced<Partial<RootState>, AppDispatch> } => {
  const { Wrapper, store } = getReduxStoreWithActionsListener({
    ...INITIAL_STORE_STATE_MOCK,
    ...initialStoreState,
  });

  return (
    render(
      <Wrapper>
        <BioEntityDrawer />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('BioEntityDrawer - component', () => {
  beforeEach(() => {
    jest.resetAllMocks();
    jest.clearAllMocks();
  });

  describe("when there's NO matching bioEntity", () => {
    beforeEach(() =>
      renderComponent({
        bioEntity: {
          data: [],
          loading: 'succeeded',
          error: { message: '', name: '' },
        },
        drawer: DRAWER_INITIAL_STATE,
      }),
    );

    it('should not show drawer content', () => {
      expect(screen.queryByText('Compartment:')).toBeNull();
      expect(screen.queryByText('Full name:')).toBeNull();
      expect(screen.queryByText('Annotations:')).toBeNull();
      expect(screen.queryByText('Source:')).toBeNull();
    });
  });
  describe('when there IS a matching bioEntity', () => {
    const { bioEntity } = bioEntitiesContentFixture[FIRST_ARRAY_ELEMENT];

    it('should show drawer header', () => {
      renderComponent({
        bioEntity: {
          data: [
            {
              searchQueryElement: '',
              loading: 'succeeded',
              error: { name: '', message: '' },
              data: bioEntitiesContentFixture,
            },
          ],
          loading: 'succeeded',
          error: { message: '', name: '' },
        },
        drawer: {
          ...DRAWER_INITIAL_STATE,
          bioEntityDrawerState: {
            bioentityId: bioEntity.id,
            drugs: {},
            chemicals: {},
          },
        },
      });

      const bioEntityType = getTypeBySBOTerm(bioEntity.sboTerm);

      expect(screen.getByText(bioEntityType, { exact: false })).toBeInTheDocument();
      expect(screen.getByText(bioEntity.name, { exact: false })).toBeInTheDocument();
    });

    it('should show drawer bioEntity full name', () => {
      renderComponent({
        bioEntity: {
          data: [
            {
              searchQueryElement: '',
              loading: 'succeeded',
              error: { name: '', message: '' },
              data: [
                {
                  ...bioEntityContentFixture,
                  bioEntity: {
                    ...bioEntityContentFixture.bioEntity,
                    fullName: 'BioEntity Full Name',
                  },
                },
              ],
            },
          ],
          loading: 'succeeded',
          error: { message: '', name: '' },
        },
        drawer: {
          ...DRAWER_INITIAL_STATE,
          bioEntityDrawerState: {
            bioentityId: bioEntityContentFixture.bioEntity.id,
            drugs: {},
            chemicals: {},
          },
        },
      });

      expect(screen.getByText('Full name:', { exact: false })).toBeInTheDocument();
      expect(screen.getByText('BioEntity Full Name', { exact: false })).toBeInTheDocument();
    });

    it("should not show drawer bioEntity full name if it doesn't exists", () => {
      renderComponent({
        bioEntity: {
          data: [
            {
              searchQueryElement: '',
              loading: 'succeeded',
              error: { name: '', message: '' },
              data: [
                {
                  ...bioEntityContentFixture,
                  bioEntity: {
                    ...bioEntityContentFixture.bioEntity,
                    fullName: null,
                  },
                },
              ],
            },
          ],
          loading: 'succeeded',
          error: { message: '', name: '' },
        },
        drawer: {
          ...DRAWER_INITIAL_STATE,
          bioEntityDrawerState: {
            bioentityId: bioEntityContentFixture.bioEntity.id,
            drugs: {},
            chemicals: {},
          },
        },
      });

      expect(screen.queryByText('Full name:')).toBeNull();
    });

    it('should show list of annotations ', () => {
      renderComponent({
        bioEntity: {
          data: [
            {
              searchQueryElement: '',
              loading: 'succeeded',
              error: { name: '', message: '' },
              data: bioEntitiesContentFixture,
            },
          ],
          loading: 'succeeded',
          error: { message: '', name: '' },
        },
        drawer: {
          ...DRAWER_INITIAL_STATE,
          bioEntityDrawerState: {
            bioentityId: bioEntity.id,
            drugs: {},
            chemicals: {},
          },
        },
      });

      expect(screen.getByText('Annotations:')).toBeInTheDocument();
      expect(screen.getByText(bioEntity.references[0].type, { exact: false })).toBeInTheDocument();
      expect(
        screen.getByText(bioEntity.references[0].resource, { exact: false }),
      ).toBeInTheDocument();
    });

    it('should display associated submaps if bio entity links to submap', () => {
      renderComponent({
        bioEntity: {
          ...BIOENTITY_INITIAL_STATE_MOCK,
          data: BIO_ENTITY_LINKING_TO_SUBMAP_DATA_MOCK,
        },
        drawer: {
          ...DRAWER_INITIAL_STATE,
          bioEntityDrawerState: {
            bioentityId: bioEntityContentFixture.bioEntity.id,
            drugs: {},
            chemicals: {},
          },
        },
        models: {
          ...MODELS_INITIAL_STATE_MOCK,
          data: MODELS_MOCK_SHORT,
        },
      });

      expect(screen.getByTestId('associated-submap')).toBeInTheDocument();
    });

    it('should display chemicals list header', () => {
      renderComponent({
        bioEntity: {
          data: [
            {
              searchQueryElement: '',
              loading: 'succeeded',
              error: { name: '', message: '' },
              data: [
                {
                  ...bioEntityContentFixture,
                  bioEntity: {
                    ...bioEntityContentFixture.bioEntity,
                    fullName: null,
                  },
                },
              ],
            },
          ],
          loading: 'succeeded',
          error: { message: '', name: '' },
        },
        drawer: {
          ...DRAWER_INITIAL_STATE,
          bioEntityDrawerState: {
            bioentityId: bioEntityContentFixture.bioEntity.id,
            drugs: {},
            chemicals: {},
          },
        },
      });

      expect(screen.getByText('Drugs for target')).toBeInTheDocument();
    });

    it('should display drugs list header', () => {
      renderComponent({
        bioEntity: {
          data: [
            {
              searchQueryElement: '',
              loading: 'succeeded',
              error: { name: '', message: '' },
              data: [
                {
                  ...bioEntityContentFixture,
                  bioEntity: {
                    ...bioEntityContentFixture.bioEntity,
                    fullName: null,
                  },
                },
              ],
            },
          ],
          loading: 'succeeded',
          error: { message: '', name: '' },
        },
        drawer: {
          ...DRAWER_INITIAL_STATE,
          bioEntityDrawerState: {
            bioentityId: bioEntityContentFixture.bioEntity.id,
            drugs: {},
            chemicals: {},
          },
        },
      });

      expect(screen.getByText('Chemicals for target', { exact: false })).toBeInTheDocument();
    });

    it('should fetch drugs on drugs for target click', () => {
      const { store } = renderComponent({
        bioEntity: {
          data: [
            {
              searchQueryElement: '',
              loading: 'succeeded',
              error: { name: '', message: '' },
              data: [
                {
                  ...bioEntityContentFixture,
                  bioEntity: {
                    ...bioEntityContentFixture.bioEntity,
                    fullName: null,
                  },
                },
              ],
            },
          ],
          loading: 'succeeded',
          error: { message: '', name: '' },
        },
        drawer: {
          ...DRAWER_INITIAL_STATE,
          bioEntityDrawerState: {
            bioentityId: bioEntityContentFixture.bioEntity.id,
            drugs: {},
            chemicals: {},
          },
        },
      });

      const button = screen.getByText('Drugs for target', { exact: false });
      act(() => {
        button.click();
      });

      expect(store.getActions()[0].type).toBe('drawer/getDrugsForBioEntityDrawerTarget/pending');
    });

    it('should fetch chemicals on chemicals for target click', () => {
      const { store } = renderComponent({
        bioEntity: {
          data: [
            {
              searchQueryElement: '',
              loading: 'succeeded',
              error: { name: '', message: '' },
              data: [
                {
                  ...bioEntityContentFixture,
                  bioEntity: {
                    ...bioEntityContentFixture.bioEntity,
                    fullName: null,
                  },
                },
              ],
            },
          ],
          loading: 'succeeded',
          error: { message: '', name: '' },
        },
        drawer: {
          ...DRAWER_INITIAL_STATE,
          bioEntityDrawerState: {
            bioentityId: bioEntityContentFixture.bioEntity.id,
            drugs: {},
            chemicals: {},
          },
        },
      });

      const button = screen.getByText('Chemicals for target', { exact: false });
      act(() => {
        button.click();
      });

      expect(store.getActions()[0].type).toBe(
        'drawer/getChemicalsForBioEntityDrawerTarget/pending',
      );
    });
  });
});
