/* eslint-disable no-magic-numbers */
import { Feature } from 'ol';
import VectorLayer from 'ol/layer/Vector';
import VectorSource from 'ol/source/Vector';
import { useEffect, useMemo } from 'react';
import { usePointToProjection } from '@/utils/map/usePointToProjection';
import MapElement from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/MapElement';
import { useSelector } from 'react-redux';
import {
  arrowTypesSelector,
  bioShapesSelector,
  lineTypesSelector,
} from '@/redux/shapes/shapes.selectors';
import { MapInstance } from '@/types/map';
import { modelElementsSelector } from '@/redux/modelElements/modelElements.selector';
import { currentModelIdSelector } from '@/redux/models/models.selectors';
import { getModelElements } from '@/redux/modelElements/modelElements.thunks';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import CompartmentSquare from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/CompartmentSquare';
import CompartmentCircle from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/CompartmentCircle';
import Glyph from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/Glyph';
import CompartmentPathway from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/elements/CompartmentPathway';
import Reaction from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/reaction/Reaction';
import { newReactionsDataSelector } from '@/redux/newReactions/newReactions.selectors';
import { getNewReactions } from '@/redux/newReactions/newReactions.thunks';
import { VECTOR_MAP_LAYER_TYPE } from '@/components/Map/MapViewer/MapViewerVector/MapViewerVector.constants';
import {
  getOverlayOrderSelector,
  overlayBioEntitiesForCurrentModelSelector,
} from '@/redux/overlayBioEntity/overlayBioEntity.selector';
import { groupBy } from '@/utils/array/groupBy';
import { useGetOverlayColor } from '@/components/Map/MapViewer/utils/config/overlaysLayer/useGetOverlayColor';
import { useAppSelector } from '@/redux/hooks/useAppSelector';
import getOverlays from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/overlay/getOverlays';
import LineOverlay from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/overlay/LineOverlay';
import { markersSufraceOfCurrentMapDataSelector } from '@/redux/markers/markers.selectors';
import { parseSurfaceMarkersToBioEntityRender } from '@/components/Map/MapViewer/utils/config/overlaysLayer/parseSurfaceMarkersToBioEntityRender';
import MarkerOverlay from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/overlay/MarkerOverlay';
import processModelElements from '@/components/Map/MapViewer/MapViewerVector/utils/config/reactionsLayer/processModelElements';
import useDebouncedValue from '@/utils/useDebouncedValue';

export const useOlMapReactionsLayer = ({
  mapInstance,
}: {
  mapInstance: MapInstance;
}): VectorLayer<VectorSource<Feature>> => {
  const dispatch = useAppDispatch();

  const currentModelId = useSelector(currentModelIdSelector);
  const modelElements = useSelector(modelElementsSelector);
  const modelReactions = useSelector(newReactionsDataSelector);
  const shapes = useSelector(bioShapesSelector);
  const lineTypes = useSelector(lineTypesSelector);
  const arrowTypes = useSelector(arrowTypesSelector);
  const overlaysOrder = useSelector(getOverlayOrderSelector);
  const currentMarkers = useAppSelector(markersSufraceOfCurrentMapDataSelector);
  const markersRender = parseSurfaceMarkersToBioEntityRender(currentMarkers);
  const bioEntities = useAppSelector(overlayBioEntitiesForCurrentModelSelector);
  const debouncedBioEntities = useDebouncedValue(bioEntities, 2000);
  const { getOverlayBioEntityColorByAvailableProperties } = useGetOverlayColor();

  const pointToProjection = usePointToProjection();

  useEffect(() => {
    if (currentModelId) {
      dispatch(getModelElements(currentModelId));
      dispatch(getNewReactions(currentModelId));
    }
  }, [currentModelId, dispatch]);

  const groupedElementsOverlays = useMemo(() => {
    const elementsBioEntitesOverlay = debouncedBioEntities.filter(
      bioEntity => bioEntity.type !== 'line',
    );
    const grouped = groupBy(elementsBioEntitesOverlay, bioEntity => bioEntity.id.toString());
    return getOverlays(grouped, getOverlayBioEntityColorByAvailableProperties);
  }, [debouncedBioEntities, getOverlayBioEntityColorByAvailableProperties]);

  const linesOverlays = useMemo(() => {
    return bioEntities.filter(bioEntity => bioEntity.type === 'line');
  }, [bioEntities]);

  const linesOverlaysFeatures = useMemo(() => {
    return linesOverlays.map(lineOverlay => {
      return new LineOverlay({
        lineOverlay,
        getOverlayColor: getOverlayBioEntityColorByAvailableProperties,
        pointToProjection,
        mapInstance,
      }).lineFeature;
    });
  }, [
    getOverlayBioEntityColorByAvailableProperties,
    linesOverlays,
    mapInstance,
    pointToProjection,
  ]);

  const markerOverlaysFeatures = useMemo(() => {
    return markersRender.map(marker => {
      return new MarkerOverlay({
        markerOverlay: marker,
        getOverlayColor: getOverlayBioEntityColorByAvailableProperties,
        pointToProjection,
        mapInstance,
      }).markerFeature;
    });
  }, [
    getOverlayBioEntityColorByAvailableProperties,
    mapInstance,
    markersRender,
    pointToProjection,
  ]);

  const reactions = useMemo(() => {
    return modelReactions.map(reaction => {
      const reactionShapes = shapes && shapes[reaction.sboTerm];
      if (!reactionShapes) {
        return [];
      }
      const reactionObject = new Reaction({
        id: reaction.id,
        line: reaction.line,
        products: reaction.products,
        reactants: reaction.reactants,
        modifiers: reaction.modifiers,
        operators: reaction.operators,
        zIndex: reaction.z,
        lineTypes,
        arrowTypes,
        shapes: reactionShapes,
        pointToProjection,
        mapInstance,
      });
      return reactionObject.features;
    });
  }, [arrowTypes, lineTypes, mapInstance, modelReactions, pointToProjection, shapes]);

  const elements: Array<
    MapElement | CompartmentCircle | CompartmentSquare | CompartmentPathway | Glyph
  > = useMemo(() => {
    if (!modelElements || !shapes) {
      return [];
    }
    return processModelElements(
      modelElements,
      shapes,
      lineTypes,
      groupedElementsOverlays,
      overlaysOrder,
      getOverlayBioEntityColorByAvailableProperties,
      mapInstance,
      pointToProjection,
    );
  }, [
    modelElements,
    shapes,
    pointToProjection,
    mapInstance,
    lineTypes,
    groupedElementsOverlays,
    overlaysOrder,
    getOverlayBioEntityColorByAvailableProperties,
  ]);

  const features = useMemo(() => {
    const reactionsFeatures = reactions.flat();
    const elementsFeatures = elements.map(element => element.feature);
    return [
      ...reactionsFeatures,
      ...elementsFeatures,
      ...linesOverlaysFeatures,
      ...markerOverlaysFeatures,
    ];
  }, [elements, linesOverlaysFeatures, markerOverlaysFeatures, reactions]);

  const vectorSource = useMemo(() => new VectorSource(), []);

  useEffect(() => {
    vectorSource.clear();
    vectorSource.addFeatures(features);
  }, [features, vectorSource]);

  return useMemo(() => {
    const vectorLayer = new VectorLayer({
      source: vectorSource,
    });
    vectorLayer.set('type', VECTOR_MAP_LAYER_TYPE);
    return vectorLayer;
  }, [vectorSource]);
};
