/* eslint-disable no-magic-numbers */
import Polygon from 'ol/geom/Polygon';
import { Style } from 'ol/style';
import Feature, { FeatureLike } from 'ol/Feature';
import { MultiPolygon } from 'ol/geom';
import { UsePointToProjectionResult } from '@/utils/map/usePointToProjection';
import {
  HorizontalAlign,
  VerticalAlign,
} from '@/components/Map/MapViewer/MapViewerVector/MapViewerVector.types';
import { MapInstance } from '@/types/map';
import getTextStyle from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/text/getTextStyle';
import { rgbToHex } from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/style/rgbToHex';
import getTextCoords from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/text/getTextCoords';
import { Color } from '@/types/models';
import {
  COMPLEX_SBO_TERM,
  MAP_ELEMENT_TYPES,
} from '@/components/Map/MapViewer/MapViewerVector/MapViewerVector.constants';

export interface BaseMapElementProps {
  type: string;
  sboTerm?: string;
  id: number;
  x: number;
  y: number;
  width: number;
  height: number;
  zIndex: number;
  text: string;
  fontSize: number;
  nameX: number;
  nameY: number;
  nameWidth: number;
  nameHeight: number;
  fontColor: Color;
  nameVerticalAlign: VerticalAlign;
  nameHorizontalAlign: HorizontalAlign;
  fillColor: Color;
  borderColor: Color;
  pointToProjection: UsePointToProjectionResult;
}

export default abstract class BaseMultiPolygon {
  type: string;

  sboTerm: string | undefined;

  id: number;

  x: number;

  y: number;

  width: number;

  height: number;

  zIndex: number;

  text: string;

  fontSize: number;

  nameX: number;

  nameY: number;

  nameWidth: number;

  nameHeight: number;

  fontColor: Color;

  nameVerticalAlign: VerticalAlign;

  nameHorizontalAlign: HorizontalAlign;

  fillColor: Color;

  borderColor: Color;

  polygons: Array<Polygon> = [];

  styles: Array<Style> = [];

  feature: Feature = new Feature();

  pointToProjection: UsePointToProjectionResult;

  constructor({
    type,
    sboTerm,
    id,
    x,
    y,
    width,
    height,
    zIndex,
    text,
    fontSize,
    nameX,
    nameY,
    nameWidth,
    nameHeight,
    fontColor,
    nameVerticalAlign,
    nameHorizontalAlign,
    fillColor,
    borderColor,
    pointToProjection,
  }: BaseMapElementProps) {
    this.type = type;
    this.sboTerm = sboTerm;
    this.id = id;
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
    this.zIndex = zIndex;
    this.text = text;
    this.fontSize = fontSize;
    this.nameX = nameX;
    this.nameY = nameY;
    this.nameWidth = nameWidth;
    this.nameHeight = nameHeight;
    this.fontColor = fontColor;
    this.nameVerticalAlign = nameVerticalAlign;
    this.nameHorizontalAlign = nameHorizontalAlign;
    this.fillColor = fillColor;
    this.borderColor = borderColor;
    this.pointToProjection = pointToProjection;
  }

  protected abstract createPolygons(): void;

  protected drawText(): void {
    if (this.text) {
      const textCoords = getTextCoords({
        x: this.nameX,
        y: this.nameY,
        width: this.nameWidth,
        height: this.nameHeight,
        fontSize: this.fontSize,
        verticalAlign: this.nameVerticalAlign,
        horizontalAlign: this.nameHorizontalAlign,
        pointToProjection: this.pointToProjection,
      });
      const textPolygon = new Polygon([[textCoords, textCoords]]);
      textPolygon.set('type', MAP_ELEMENT_TYPES.TEXT);
      textPolygon.set('text', this.text);
      textPolygon.set('fontSize', this.fontSize);
      const textStyle = getTextStyle({
        text: this.text,
        fontSize: this.fontSize,
        color: rgbToHex(this.fontColor),
        zIndex: this.zIndex,
        horizontalAlign: this.nameHorizontalAlign,
      });
      textStyle.setGeometry(textPolygon);
      this.styles.push(textStyle);
      this.polygons.push(textPolygon);
    }
  }

  protected drawMultiPolygonFeature(mapInstance: MapInstance): void {
    this.feature = new Feature({
      geometry: new MultiPolygon(this.polygons),
      getScale: (resolution: number): number => {
        const maxZoom = mapInstance?.getView().get('originalMaxZoom');
        if (maxZoom) {
          const minResolution = mapInstance?.getView().getResolutionForZoom(maxZoom);
          if (minResolution) {
            return minResolution / resolution;
          }
        }
        return 1;
      },
      id: this.id,
      type: this.type,
    });

    this.feature.setStyle(this.getStyle.bind(this));
  }

  protected getStyle(feature: FeatureLike, resolution: number): Style | Array<Style> | void {
    const styles: Array<Style> = [];
    const getScale = feature.get('getScale');
    let scale = 1;
    if (getScale instanceof Function) {
      scale = getScale(resolution);
    }

    let type: string;
    let fontSize: number;
    let lineWidth: number;
    let text: string;

    this.styles.forEach(style => {
      const styleGeometry = style.getGeometry();
      if (styleGeometry instanceof Polygon) {
        type = styleGeometry.get('type');
        text = styleGeometry.get('text');
        fontSize = styleGeometry.get('fontSize');
        lineWidth = styleGeometry.get('lineWidth');
      }
      if (
        [MAP_ELEMENT_TYPES.MODIFICATION, MAP_ELEMENT_TYPES.TEXT].includes(type) &&
        scale * fontSize <= 4
      ) {
        return;
      }

      const clonedStyle = style.clone();
      const textStyle = clonedStyle.getText();
      const strokeStyle = clonedStyle.getStroke();
      if (type === 'text' && textStyle) {
        textStyle.setScale(scale);
        textStyle.setText(text);
      }
      if (strokeStyle && lineWidth) {
        if (lineWidth * scale < 0.08 && this.sboTerm !== COMPLEX_SBO_TERM) {
          clonedStyle.setStroke(null);
        } else {
          strokeStyle.setWidth(lineWidth * scale);
          const lineDash = strokeStyle.getLineDash();
          if (lineDash) {
            const newLineDash = lineDash.map(width => width * scale);
            strokeStyle.setLineDash(newLineDash);
          }
        }
      }
      styles.push(clonedStyle);
    });
    return styles;
  }
}
