/* eslint-disable no-magic-numbers */
import { apiPath } from '@/redux/apiPath';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkNewAPIResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import { unwrapResult } from '@reduxjs/toolkit';
import { ModelElementsState } from '@/redux/modelElements/modelElements.types';
import modelElementsReducer from '@/redux/modelElements/modelElements.slice';
import { getModelElements } from '@/redux/modelElements/modelElements.thunks';
import { modelElementsFixture } from '@/models/fixtures/modelElementsFixture';

const mockedAxiosClient = mockNetworkNewAPIResponse();

const INITIAL_STATE: ModelElementsState = {
  data: null,
  loading: 'idle',
  error: { name: '', message: '' },
};

describe('model elements reducer', () => {
  let store = {} as ToolkitStoreWithSingleSlice<ModelElementsState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('modelElements', modelElementsReducer);
  });

  it('should match initial state', () => {
    const action = { type: 'unknown' };

    expect(modelElementsReducer(undefined, action)).toEqual(INITIAL_STATE);
  });

  it('should update store after successful getModelElements query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getModelElements(0))
      .reply(HttpStatusCode.Ok, modelElementsFixture);

    const { type } = await store.dispatch(getModelElements(0));
    const { data, loading, error } = store.getState().modelElements;

    expect(type).toBe('vectorMap/getModelElements/fulfilled');
    expect(loading).toEqual('succeeded');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual(modelElementsFixture);
  });

  it('should update store after failed getModelElements query', async () => {
    mockedAxiosClient.onGet(apiPath.getModelElements(0)).reply(HttpStatusCode.NotFound, []);

    const action = await store.dispatch(getModelElements(0));
    const { data, loading, error } = store.getState().modelElements;

    expect(action.type).toBe('vectorMap/getModelElements/rejected');
    expect(() => unwrapResult(action)).toThrow(
      "Failed to fetch model elements: The page you're looking for doesn't exist. Please verify the URL and try again.",
    );
    expect(loading).toEqual('failed');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual(null);
  });

  it('should update store on loading getModelElements query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getModelElements(0))
      .reply(HttpStatusCode.Ok, modelElementsFixture);

    const modelElementsPromise = store.dispatch(getModelElements(0));

    const { data, loading } = store.getState().modelElements;
    expect(data).toEqual(null);
    expect(loading).toEqual('pending');

    modelElementsPromise.then(() => {
      const { data: dataPromiseFulfilled, loading: promiseFulfilled } =
        store.getState().modelElements;

      expect(dataPromiseFulfilled).toEqual(modelElementsFixture);
      expect(promiseFulfilled).toEqual('succeeded');
    });
  });
});
